//
//  TVSSplitViewController.m
//  TableViewSpliter
//
//  Created by narita on 2016/07/29.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "TVSSplitViewController.h"
#import "TVSViewController.h"

@interface TVSSplitViewController ()

@end

@implementation TVSSplitViewController

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

#pragma mark -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark restorableState methods

- (void) encodeRestorableStateWithCoder:(NSCoder *)coder
{
    [super encodeRestorableStateWithCoder:coder];
    
    // アクティブなPaneを探して記録
    NSSplitViewItem* theResultItem = nil;
    NSResponder*     theResponder  = self.view.window.firstResponder;
    
    while((theResponder = [theResponder nextResponder]))
    {
        for(NSSplitViewItem* i in self.splitViewItems)
        {
            if( theResponder == i.viewController.view )
            {
                theResultItem = i;
                theResponder = nil;
                break;
            }            
        }
    }
    
    if( theResultItem != nil )
    {
        // activeなPaneのidentifierを保存
        [coder encodeObject:theResultItem.viewController.view.identifier
                     forKey:@"activePaneIdentifierInSplitView"];
    }
}

- (void) restoreStateWithCoder:(NSCoder *)coder
{
    [super restoreStateWithCoder:coder];
    
    // activeなPaneの再生
    NSString* theActiveIdentifier = [coder decodeObjectForKey:@"activePaneIdentifierInSplitView"];
    
    if( theActiveIdentifier != nil )
    {
        for(NSSplitViewItem* i in self.splitViewItems)
        {
            if( [i.viewController.view.identifier isEqualToString:theActiveIdentifier] )
            {
                // 既存のfirstResponderの再生より後に選択させる
                [[NSOperationQueue mainQueue] addOperationWithBlock:^{
                    [i.viewController.view becomeFirstResponder];
                }];
                break;
            }            
        }
    }
}

#pragma mark -
#pragma mark awakeFromNib-like methods

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    NSSplitViewItem*  theItem = [self makeNewSplitViewItem];
    [self addSplitViewItem:theItem];
    
    // Scrollerに名前を付けて、restore stateに対応する
    [self nameSpliterPanes];
}


- (void) viewWillAppear
{
    [super viewWillAppear];
    
    // restorableStateKeyPathsでself.view.window.firstResponderの変化が取得できない。落ちる。
    // 仕方がないので、KVOで取得して対処している。
    [self.view.window addObserver:self 
                       forKeyPath:@"firstResponder"
                          options:0 
                          context:(__bridge void*)self];
}

- (void) viewWillDisappear
{
    [self.view.window removeObserver:self 
                          forKeyPath:@"firstResponder" 
                             context:(__bridge void*)self];
    
    [super viewWillDisappear];
}

#pragma mark -
#pragma mark action methods

- (void) createPaneWithNumber:(NSInteger)inNumber
{
    if( inNumber != 0 )
    {
        // 既存のspliterを全て削除する
        for(NSSplitViewItem* i in self.splitViewItems)
        {
            [self removeSplitViewItem:i];
        }
        
        // 個数分分割して    
        while(inNumber != [self countOfDivider]+1)
        {
            NSSplitViewItem*  theItem = [self makeNewSplitViewItem];
            [self addSplitViewItem:theItem];
        }
    }
    
    // Scrollerに名前を付けて、restore stateに対応する
    [self nameSpliterPanes];
}

- (NSSplitViewItem*) makeNewSplitViewItem
{
    NSStoryboard*     theStoryboard;
    NSViewController* theViewController;
    NSSplitViewItem*  theItem;
    
    theStoryboard     = [NSStoryboard storyboardWithName:@"MainViewController" bundle:nil];
    theViewController = [theStoryboard instantiateInitialController];
    theItem           = [NSSplitViewItem splitViewItemWithViewController:theViewController];
    
    return theItem;
}

- (void) insertNewPaneAfter:(TVSViewController*)inViewController
{
    NSSplitViewItem* theCurrentItem = [self splitViewItemForViewController:inViewController];
    NSInteger        theIndex       = [self.splitViewItems indexOfObject:theCurrentItem];
    
    NSSplitViewItem* theNewItem = [self makeNewSplitViewItem];

    // 区切り線の位置を保存して
    NSMutableArray* theArray = [[self spliterPositionsArray] mutableCopy];
    
    
    // スクロール位置の調整の準備
    NSScrollView* theCurrentScrollerView = (NSScrollView*)theCurrentItem.viewController.view;
    NSRect        theCurrentRect         = theCurrentScrollerView.contentView.bounds;
    
    // 区切り線を挿入
    [self insertSplitViewItem:theNewItem
                      atIndex:theIndex+1];

    // 挿入後に区切り線の位置を調整して復帰
    CGFloat thePosition_0 = theIndex != 0 ? [[theArray objectAtIndex:theIndex-1] floatValue] : 0;
    CGFloat thePosition_1 = [[theArray objectAtIndex:theIndex] floatValue];
    CGFloat thePosition_2 = roundf((thePosition_1 - thePosition_0)/ 2.0) + thePosition_0;
    [theArray insertObject:@(thePosition_2) atIndex:theIndex];
    
    [self setSpliterPositionsArray:theArray];

    
    // 新しく作ったviewのスクロール位置を調整
    NSScrollView* theNewScrollerView = (NSScrollView*)theNewItem.viewController.view;

    NSRect theNewRect = theNewScrollerView.contentView.bounds;
    theNewRect.origin.y = theCurrentRect.origin.y + theCurrentRect.size.height / 2;
    
    // 理由は不明だが、自動でスクロール位置が変更されてしまうために、遅延実行している
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        theNewScrollerView.contentView.bounds = theNewRect;
    }];
    
    [self invalidateRestorableState];
    
    // Scrollerに名前を付けて、restore stateに対応する
    [self nameSpliterPanes];
}


// 与えられたTSViewControllerを削除する
- (void) removePane:(TVSViewController*)inViewController
{
    if( self.splitViewItems != nil && self.splitViewItems.count > 1)
    {
        NSSplitViewItem* theItem  = [self splitViewItemForViewController:inViewController];
        NSInteger        theIndex = [self.splitViewItems indexOfObject:theItem];

        // 区切り線の位置を保存して
        NSMutableArray* theArray = [[self spliterPositionsArray] mutableCopy];

        // theItem.viewControllerがアクティブなレスポンダーチェインに属しているならば、
        // 削除により、firstResponderのポイント先がなくなるので変更する
        BOOL theResetFirstResponder = NO;
        
        NSResponder* theResponder = self.view.window.firstResponder;
        
        while( theResponder )
        {
            if( theResponder == theItem.viewController )
            {
                theResetFirstResponder = YES;
                break;
            }
            
            theResponder = [theResponder nextResponder];
        }        
        
        // TODO:スクロール位置の調整とアニメーション
        [self removeSplitViewItem:theItem];
        
        // 区切り線の位置を調整して復帰
        [theArray removeObjectAtIndex:MAX(0, theIndex-1)];
        [self setSpliterPositionsArray:theArray];

        //削除するPaneにフォーカスを変更
        if(theResetFirstResponder)
        {
            [self makeFirstResponderAtIndex:MAX(theIndex-1, 0)];
        }
        
        [self invalidateRestorableState];
        
        // Scrollerに名前を付けて、restore stateに対応する
        [self nameSpliterPanes];
    }
}

// 指定したIndexへレスポンダーを移動
- (void) makeFirstResponderAtIndex:(NSInteger)inIndex
{
    NSInteger theIndex = MIN(inIndex, self.splitViewItems.count -1);
    
    NSSplitViewItem* theItem  = [self.splitViewItems objectAtIndex:theIndex];
    
    [self.view.window makeFirstResponder:theItem.viewController.view];
}

#pragma mark -
#pragma mark delegate/datasource methods

// 分割線を移動する時にscrollViewの中身をそれに合わせてスクロールさせる
static CGFloat sOldPosition = 0;

- (void)splitViewWillResizeSubviews:(NSNotification *)notification
{
    NSNumber* theBOOL = [notification.userInfo valueForKey:@"NSSplitViewUserResizeKey"];
    
    //    NSSplitViewUserResizeKey == YESの場合にのみ処理をする
    if( theBOOL.boolValue == YES )
    {
        NSNumber* theIndexObject = [notification.userInfo valueForKey:@"NSSplitViewDividerIndex"];
        NSInteger theIndex = theIndexObject.integerValue;
        sOldPosition = [self positionOfDividerAtIndex:theIndex];
    }
}

- (void)splitViewDidResizeSubviews:(NSNotification *)notification
{
    NSNumber* theBOOL = [notification.userInfo valueForKey:@"NSSplitViewUserResizeKey"];
    
    //    NSSplitViewUserResizeKey == YESの場合にのみ処理をする
    if( theBOOL.boolValue == YES )
    {
        NSNumber* theIndexObject = [notification.userInfo valueForKey:@"NSSplitViewDividerIndex"];
        NSInteger theIndex = theIndexObject.integerValue;
        
        CGFloat theCurrentPosition = [self positionOfDividerAtIndex:theIndex];
    
        NSSplitViewItem* theItem = [self.splitViewItems objectAtIndex:theIndex+1];
        NSScrollView* theScrollerView = (NSScrollView*)theItem.viewController.view;
        
        NSRect theRect = theScrollerView.contentView.bounds;
        theRect.origin.y += theCurrentPosition - sOldPosition;
        
        theScrollerView.contentView.bounds = theRect;
    }
}

#pragma mark -
#pragma mark accessor methods (in pairs)

- (NSArray*) spliterPositionsArray
{
    NSMutableArray* theArray = [NSMutableArray array];
    
    for(NSInteger i = 0; i <= [self countOfDivider]; i++)
    {
        CGFloat thePosition = [self positionOfDividerAtIndex:i];
        
        [theArray addObject:@(thePosition)];
    }
    
    return [theArray copy];
}

- (void) setSpliterPositionsArray:(NSArray*)inArray
{
    // 幅を揃える
    for(NSInteger i = 0; i < [inArray count]; i++)
    {
        CGFloat thePosition = [[inArray objectAtIndex:i] floatValue];
        
        [self.splitView setPosition:thePosition
                   ofDividerAtIndex:i];
    }
}

// Scrollerに名前を付けて、restore stateに対応する
- (void) nameSpliterPanes
{
    for(NSSplitViewItem* theItem in self.splitViewItems)
    {
        NSInteger theIndex = [self.splitViewItems indexOfObject:theItem];
        
        theItem.viewController.view.identifier = 
        [NSString stringWithFormat:@"SplitScroller_%@", @(theIndex)];
    }
}

#pragma mark -
#pragma mark KVO methods

- (void)observeValueForKeyPath:(NSString *)keyPath 
                      ofObject:(id)object 
                        change:(NSDictionary *)change 
                       context:(void *)context 
{
    if( context == (__bridge void*) self )
    {
        if( [object isEqual:self.view.window] )
        {
            if( [keyPath isEqualToString:@"firstResponder"] )
            {
                [self invalidateRestorableState];
            }
        }
    }
    else
    {
        [super observeValueForKeyPath:keyPath
                             ofObject:object 
                               change:change 
                              context:context];
    }
}

#pragma mark -
#pragma mark Utility methods

- (NSInteger) countOfDivider
{
    NSInteger theCount = self.splitViewItems.count;
    
    return (theCount == 0 ) ? 0 : theCount - 1;
}

- (void)setPosition:(CGFloat)position ofDividerAtIndex:(NSInteger)dividerIndex
{
    NSAssert( dividerIndex >= 0, nil);
    NSAssert( dividerIndex < [self countOfDivider], nil);
    
    [self.splitView setPosition:position
               ofDividerAtIndex:dividerIndex];
}

- (CGFloat)positionOfDividerAtIndex:(NSInteger)dividerIndex
{
    NSAssert( dividerIndex >= 0, nil);
    NSAssert( dividerIndex <= [self countOfDivider], nil);
    
    CGFloat thePosition = 0;
    
    for(NSInteger i = 0; i <= dividerIndex; i++)
    {
        NSSplitViewItem* theItem = [self.splitViewItems objectAtIndex:i];
        NSRect theFrame = theItem.viewController.view.frame;
        
        thePosition += theFrame.size.height;
        
        // 2個目のitem以降にはdividerThicknessを含める
        if( i != 0)
        {
            thePosition += self.splitView.dividerThickness;
        }
    }
    
    return thePosition;
}

@end
