//
//  TVSViewController.m
//  TableViewSpliter
//
//  Created by narita on 2016/07/28.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "TVSViewController.h"
#import "TVSSplitViewController.h"
#import "TVSplitableScrollView.h"

@implementation TVSViewController

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

#pragma mark -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark restorableState methods

#if 0

// encode&restore方式とは排他的に行う

+ (NSArray *)restorableStateKeyPaths
{
    return [[super restorableStateKeyPaths] arrayByAddingObjectsFromArray:@[ @"color" ]];
}

#else

- (void) encodeRestorableStateWithCoder:(NSCoder *)coder
{
    [super encodeRestorableStateWithCoder:coder];
    
    NSData* theData=[NSArchiver archivedDataWithRootObject:self.color];

    [coder encodeObject:theData forKey:@"color"];
}

- (void) restoreStateWithCoder:(NSCoder *)coder
{
    [super restoreStateWithCoder:coder];
    
    NSData* theData = [coder decodeObjectForKey:@"color"];
    
    if (theData != nil)
    {
        self.color =(NSColor *)[NSUnarchiver unarchiveObjectWithData:theData];
    }
}

#endif


#pragma mark -
#pragma mark awakeFromNib-like methods

- (void)viewDidLoad
{
    [super viewDidLoad];
    
}

- (void) viewWillAppear
{
    [super viewWillAppear];

}

- (void) viewDidAppear
{
    [super viewDidAppear];

    self.representedObject = self.view.window.windowController.document;
}

#pragma mark -
#pragma mark action methods

- (IBAction) newPane:(id)sender
{
    TVSSplitViewController* theVC = (TVSSplitViewController*)self.parentViewController;
    
    [theVC insertNewPaneAfter:self];
}

- (BOOL) canNewPane:(id)sender
{
    // 最小限高さより小さい場合は、分割できない
    // ( sender.frame.size.height - self.dividerThickness ) / 2
    
    // 最小高さの値は、SplitViewItemの中のViewControllerが持っているようにする
    // ViewControllerは特定のカテゴリに属するか、viewController.viewの持つautoLayoutの値に従うかも
    
    return YES;
}

- (IBAction) deletePane:(id)sender
{
    TVSSplitViewController* theVC = (TVSSplitViewController*)self.parentViewController;
    
    [theVC removePane:self];
}

- (BOOL) canDeletePane:(id)sender
{
    // subviewが最後の1つならば、close出来ない
    // splitViewItemsの数が1以下であればこれ以上は削除できない

    TVSSplitViewController* theVC = (TVSSplitViewController*)self.parentViewController;

    if( theVC.splitViewItems.count > 1 )
    {
        return YES;
    }
    else
    {
        return NO;
    }
}


#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

- (void)setRepresentedObject:(id)representedObject
{
    [super setRepresentedObject:representedObject];
}

#if 1
// encode & decode方式の場合はこちら
@synthesize color = _color;

- (void) setColor:(NSColor*) inColor
{
    @synchronized(self)
    {
        _color = [inColor copy];
    }

    [self invalidateRestorableState];
}

- (NSColor*) color
{
    @synchronized(self)
    {
        return _color;
    }
}
#endif

- (NSAttributedString*) string
{
    NSSplitViewController* theVC = (TVSSplitViewController*)self.parentViewController;
    NSSplitViewItem* theItem = [theVC splitViewItemForViewController:self];
    
    NSString *theString = [NSString stringWithFormat:@"%@ => %@", self, @([theVC.splitViewItems indexOfObject:theItem])];
    
    NSAttributedString* theResult = [[NSAttributedString alloc] initWithString:theString];
    
    return theResult;
}

#pragma mark -
#pragma mark Utility methods


@end
