//
//  TVSWindowController.m
//  TableViewSpliter
//
//  Created by narita on 2016/07/28.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "TVSWindowController.h"
#import "TVSSplitViewController.h"

@interface TVSWindowController ()

// 削除後の選択範囲を再設定するために、選択しているObjectを保持する配列
@property (strong) NSArray* privateSelectedObjects;

@end

@implementation TVSWindowController

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods


#pragma mark -
#pragma mark dealloc


#pragma mark -
#pragma mark NSCopying, hash, isEqual:


#pragma mark -
#pragma mark NSCoding methods


#pragma mark -
#pragma mark restorableState methods

+ (NSArray *)restorableStateKeyPaths
{
    return [[super restorableStateKeyPaths] arrayByAddingObjectsFromArray:@[ @"indexes" ]];
}

- (void) encodeRestorableStateWithCoder:(NSCoder *)coder
{
    TVSSplitViewController* theSplitViewController = (TVSSplitViewController*) self.contentViewController;

    [super encodeRestorableStateWithCoder:coder];
    
    [coder encodeObject:[NSNumber numberWithInteger:theSplitViewController.splitViewItems.count]
              forKey:@"numOfSplit"];
    
    [coder encodeObject:self.indexes forKey:@"indexes"];
}

- (void) restoreStateWithCoder:(NSCoder *)coder
{
    [super restoreStateWithCoder:coder]; 
    
    // 選択範囲の再生    
    NSIndexSet* theIndexSet = [coder decodeObjectForKey:@"indexes"];
    
    // NSTableView選択範囲の再生タイミングをずらさないとできなかった。
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        self.indexes = [theIndexSet mutableCopy];        
    }];
}


- (void) prebuildForRestorableStateWithWindowCoder:(NSCoder*)coder
{
    TVSSplitViewController* theSplitViewController = (TVSSplitViewController*) self.contentViewController;
        
    // restore state のコードが走り出す前に、View構造を構築する必要がある。
    NSInteger theSplitNumber = [[coder decodeObjectForKey:@"numOfSplit"] integerValue];

    [theSplitViewController createPaneWithNumber:theSplitNumber];
}


#pragma mark -
#pragma mark awakeFromNib-like methods

- (void)windowDidLoad
{
    [super windowDidLoad];
    
    // Implement this method to handle any initialization after your window controller's window has been loaded from its nib file.
    
    self.indexes = [NSMutableIndexSet indexSet];
    
    [self addObserver:self 
           forKeyPath:@"document.model"
              options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld | NSKeyValueObservingOptionPrior
              context:(__bridge void*)self];
}


#pragma mark -
#pragma mark KVO methods

- (void)observeValueForKeyPath:(NSString *)keyPath 
                      ofObject:(id)object 
                        change:(NSDictionary *)change 
                       context:(void *)context 
{
    if( context == (__bridge void*) self )
    {
        if( [object isEqual:self] )
        {
            if( [keyPath isEqualToString:@"document.model"] )
            {                
                // 削除である
                if([[change objectForKey:NSKeyValueChangeKindKey] integerValue] == NSKeyValueChangeRemoval)
                {
                    if( [[change objectForKey:NSKeyValueChangeNotificationIsPriorKey] boolValue] == YES )
                    {
                        // before
                        NSLog(@"before %@", change);

                        // 削除する予定のIndexSet
                        NSIndexSet* theWillDeleteIndexSet = [change objectForKey:NSKeyValueChangeIndexesKey];                    
                        
                        // 選択しているIndexSet
                        NSIndexSet* theSelectedIndexSet = [self.indexes copy];
                        
                        // 選択中のindexSetから削除分を引く
                        NSMutableIndexSet* theDeletedIndexSet = [theSelectedIndexSet mutableCopy];
                        [theDeletedIndexSet removeIndexes:theWillDeleteIndexSet];
                        
                        if( theDeletedIndexSet.count == 0 )
                        {
                            // 残りがない場合は選択しない(?) UI的に正しいかはまた別
                            self.privateSelectedObjects = [NSArray array];
                            
                        }
                        else
                        {
                            // 残りがあるならば、それが削除後の選択すべきObjectになる。
                            // objectのarrayになるのは、indexSetのindexが削除した場合にはズレるため。
                            self.privateSelectedObjects = [[self valueForKeyPath:@"document.model"] objectsAtIndexes:self.indexes];
                        }
                        
                    }
                    else
                    {                        
                        //  削除後に選択し直す。
                        NSArray* theModel = [self valueForKeyPath:@"document.model"];
                        NSMutableIndexSet* theIndexSet = [NSMutableIndexSet indexSet];
                        
                        // 選択すべきindex値をリストアップ
                        for(id i in  self.privateSelectedObjects)
                        {
                            NSUInteger theIndex = [theModel indexOfObject:i];
                            
                            if( theIndex != NSNotFound )
                            {
                                [theIndexSet addIndex:[theModel indexOfObject:i]];
                            }
                        }
                        
                        // 新しい選択位置を設定する
                        self.indexes = theIndexSet;
                        
                        // リリース
                        self.privateSelectedObjects = nil;
                    }                    
                }
                
            }
        }
    }
    else
    {
        [super observeValueForKeyPath:keyPath
                             ofObject:object 
                               change:change 
                              context:context];
    }
}

#pragma mark -
#pragma mark action methods

// 選択位置を削除
- (void) delete:(id)sender
{    
    // modelを操作するときに必ず、mutableXXXを挟むことでKVO対応する
    [[self mutableArrayValueForKeyPath:@"document.model"] removeObjectsAtIndexes:self.indexes];
}

#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

// 以下の記述は殆ど無意味。+(NSSet*)keyPathsForValuesAffecting<key>の表記は、"対多"の関係をサポートしていない。
// 代わりにaddObserver:forKeyPath:options:context:使ってupdateせよとのこと
// seeAlso https://developer.apple.com/library/content/documentation/Cocoa/Conceptual/KeyValueObserving/Articles/KVODependentKeys.html
//+ (NSSet *)keyPathsForValuesAffectingIndexedObjects
//{
//    return [NSSet setWithObject:@"indexes"];
//}
//
//- (NSArray*) indexedObjects
//{
//    NSArray* theModel = [self valueForKeyPath:@"document.model"];
//    
//    return [theModel objectsAtIndexes:self.indexes];    
//}

#pragma mark -
#pragma mark Utility methods

- (NSString *) windowTitleForDocumentDisplayName: (NSString *)displayName
{
    if ([self document] != nil)
    {
        NSString*	theDisplayName;
        NSArray*	theWinCtrls;
        
        theDisplayName = [[self document] displayName];
        theWinCtrls = [[self document] windowControllers];
        
        if([theWinCtrls count] > 1)
        {
            NSUInteger theIndex;
            NSString*  theResult;
            
            //	NSArrayのindexが０から始まるため+1する
            theIndex = [theWinCtrls indexOfObject:self] + 1;
            
            theResult = [NSString stringWithFormat:@"%@:(%@)", theDisplayName, @(theIndex)];
                        
            return theResult;
        }
        else
        {
            return [super windowTitleForDocumentDisplayName:displayName];
        }
    }
    else
    {
        return [super windowTitleForDocumentDisplayName:displayName];
    }
}

@end
